#include "Sportik.h"
#include <algorithm>
#include <iostream>
#include <fstream>
#include <iomanip>
void Sportik::to_txt() {
    std::ofstream file1(file + "_total.txt");
    if (!file1.is_open()) {
        std::cerr << "Error: unable to open file" << std::endl;
        exit(0);
    }

    file1 << std::setw(54) << " All clients " << std::endl << std::endl;
    file1 << "| id |   Client's name   | code |   activity   |     coach     | duration | started on | price |" << std::endl;

    for (const auto& x : clients) {
        file1 << "|" << std::setw(3) << x.first << " |"
              << std::setw(17) << x.second.client << "  |"
              << std::setw(4) << x.second.sport_code << "  |"
              << std::setw(12) << x.second.sport << "  |"
              << std::setw(12) << x.second.coach << "  |"
              << std::setw(10) << x.second.duration << " |"
              << std::setw(3) << x.second.start.tm_mday << "." << x.second.start.tm_mon << "." << x.second.start.tm_year << " |"
              << std::setw(6) << x.second.price << "  |"
              << std::endl;
    }

}
std::map<int, Sportsman> Sportik::find_by_client_surname(const std::string& surname){
    std::map<int, Sportsman> ans;
    for (auto x : clients)
    {
        if(x.second.client == surname){
            ans.insert(x);
        }
        std::ofstream file1(surname + "_by_surname.bin", std::ios::binary);
        if (!file1.is_open()) {
            std::cerr << "Error: unable to open file" << std::endl;
            exit(0);
        }
        for (const auto& x : ans) {
            file1.write(reinterpret_cast<const char*>(&x.second), sizeof(Sportsman));
        }
        std::cout << "All clients with surname " << surname << " are written to " << surname << "_clients.bin" << std::endl;
    }
    return ans;
}
std::map<int, Sportsman> Sportik::find_by_coach_surname(const std::string& surname){
    std::map<int, Sportsman> ans;
    for (auto x : clients)
    {
        if(x.second.coach == surname){
            ans.insert(x);
        }
        std::ofstream file1(surname + "_clients.bin", std::ios::binary);
        if (!file1.is_open()) {
            std::cerr << "Error: unable to open file" << std::endl;
            exit(0);
        }
        for (const auto& x : ans) {
            file1.write(reinterpret_cast<const char*>(&x.second), sizeof(Sportsman));
        }
        std::cout << "All clients of " << surname << "are written to " << surname << "_clients.bin" << std::endl;
    }
    return ans;
}
std::map<int, Sportsman> Sportik::find_by_sport(const std::string &sport) {
    std::map<int, Sportsman> ans;
    for (auto x : clients)
    {
        if(x.second.sport == sport){
            ans.insert(x);
        }
        std::ofstream file1(sport + ".bin", std::ios::binary);
        if (!file1.is_open()) {
            std::cerr << "Error: unable to open file" << std::endl;
            exit(0);
        }
        for (const auto& x : ans) {
            file1.write(reinterpret_cast<const char*>(&x.second), sizeof(Sportsman));
        }
        std::cout << "All clients doing " << sport << "are written to " << sport << ".bin" << std::endl;
    }
    return ans;
}

std::map<int, Sportsman> Sportik::find_by_experience_less(int exp){
    std::map<int, Sportsman> ans;
    for (auto x : clients)
    {
        if(x.second.duration < exp){
            ans.insert(x);
        }
        std::ofstream file1("less_than_" + std::to_string(exp) + "_years_of_visiting.bin", std::ios::binary);
        if (!file1.is_open()) {
            std::cerr << "Error: unable to open file" << std::endl;
            exit(0);
        }
        for (const auto& x : ans) {
            file1.write(reinterpret_cast<const char*>(&x.second), sizeof(Sportsman));
        }
        std::cout << "All clients visiting less than " << exp << " years are written to less_than_" << exp << "_years_of_visiting.bin" << std::endl;
    }
    return ans;
}
std::map<int, Sportsman> Sportik::find_by_experience_more(int exp){
    std::map<int, Sportsman> ans;
    for (auto x : clients)
    {
        if(x.second.duration > exp){
            ans.insert(x);
        }
        std::ofstream file1("more_than_" + std::to_string(exp) + "_years_of_visiting.bin", std::ios::binary);
        if (!file1.is_open()) {
            std::cerr << "Error: unable to open file" << std::endl;
            exit(0);
        }
        for (const auto& x : ans) {
            file1.write(reinterpret_cast<const char*>(&x.second), sizeof(Sportsman));
        }
        std::cout << "All clients visiting more than " << exp << " years are written to more_than_" << exp << "_years_of_visiting.bin" << std::endl;
    }
    return ans;
}
std::map<int, Sportsman> Sportik::find_by_date(int year, int month, int day){
    std::map<int, Sportsman> ans;
    for (auto x : clients)
    {
        if((x.second.start.tm_year == year)&&(x.second.start.tm_mon == month)&&(x.second.start.tm_mday == day)){
            ans.insert(x);
        }
        std::string date = std::to_string(day) + "." + std::to_string(month) + "." + std::to_string(year);
        std::ofstream file1("started_on_" + date  + ".bin", std::ios::binary);
        if (!file1.is_open()) {
            std::cerr << "Error: unable to open file" << std::endl;
            exit(0);
        }
        for (const auto& x : ans) {
            file1.write(reinterpret_cast<const char*>(&x.second), sizeof(Sportsman));
        }
        std::cout << "All clients started on " << date << " are written to started_on_" << date << ".bin" << std::endl;
    }
    return ans;
}

Sportik::Sportik(const std::map<int, Sportsman> &data) {
    version = 0;
    clients = data;
    std::cout << "Enter name for new file" << std::endl;
    std::cin >> file;
    std::ofstream fout(file, std::ios::binary);
    for(const auto& cur : clients)
        fout.write(reinterpret_cast<const char*>(&cur.second), sizeof(Sportsman));
}

void Sportik::update_name() {
    for(char c : std::to_string(version))
        file.pop_back();
    version++;
    file += std::to_string(version);
}
void Sportik::replace(const Sportsman &guest, int key) {
    if(version != 0)
        update_name();
    std::ofstream fout(file, std::ios::binary);
    clients[key] = guest;
    for(const auto& cur : clients)
        fout.write(reinterpret_cast<const char*>(&cur.second), sizeof(Sportsman));
}
void Sportik::delete_client(int key) {
    clients.erase(key);
    if(version != 0)
        update_name();
    std::ofstream fout(file, std::ios::binary);
    for(const auto& cur : clients)
        fout.write(reinterpret_cast<const char*>(&cur.second), sizeof(Sportsman));
}
void Sportik::add_created(const Sportsman &guest) {
    std::ofstream fout(file, std::ios::binary);
    fout.write(reinterpret_cast<const char*>(&guest), sizeof(Sportsman));
}
void Sportik::add_creating() {
    Sportsman guest;
    std::pair<int, Sportsman> p {ind, guest};
    clients.insert(p);
    ind++;
    std::ofstream fout(file, std::ios::binary);
    fout.write(reinterpret_cast<const char*>(&guest), sizeof(Sportsman));
}
Sportsman::Sportsman() {
    int x;
    std::cout << "Creating new client" << std::endl;
    std::cout << "Input his surname: ";
    std::cin >> client;
    std::cout << std::endl << "Input code of his sport activity here: ";
    std::cin >> sport_code;
    std::cout << std::endl << "Input his sport activity here: ";
    std::cin >> sport;
    std::cout << std::endl << "Input his coach surname: ";
    std::cin >> coach;
    std::cout << std::endl << "Input his amount of minutes ";
    std::cin >> duration;
    std::cout << std::endl << "Input year he started ";
    std::cin >> x;
    start.tm_year = x;
    std::cout << std::endl << "Input month he started ";
    std::cin >> x;
    start.tm_mon = x;
    std::cout << std::endl << "Input day he started ";
    std::cin >> x;
    start.tm_mday = x;
    std::cout << std::endl << "Input his tariff per minute ";
    std::cin >> price;
}